import serial
import time
import math

# ============================================================================
# Raspi Lobot 24-Channel Servo Driver
# ============================================================================

def getLowByte(value):
    return value.to_bytes(2,'little')[0]

def getHighByte(value):
    return value.to_bytes(2,'little')[1]

class ServoController:

    # Registers/etc.
    __FRAME_HEADER            = 0x55
    __CMD_SERVO_MOVE          = 0x03
    __CMD_ACTION_GROUP_RUN    = 0x06
    __CMD_ACTION_GROUP_STOP   = 0x07
    __CMD_ACTION_GROUP_SPEED  = 0x0B
    __CMD_GET_BATTERY_VOLTAGE = 0x0F
    __BATTERY_VOLTAGE         = 0x0F
    __ACTION_GROUP_RUNNING    = 0x06
    __ACTION_GROUP_STOPPED    = 0x07
    __ACTION_GROUP_COMPLETE   = 0x08
  
    #default possition for all servos
    lobotServos = [1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,1500]
    numOfActinGroupRunning = 0xFF
    actionGroupRunTimes = 0
    
    isGetBatteryVolt = False
    batteryVoltage = 0
    isRunning = False
  
    def __init__(self, debug=False):
        self.ser = serial.Serial('/dev/ttyS0', 9600, timeout=1)
        self.ser.flush()
        
        numOfActinGroupRunning = 0xFF
        actionGroupRunTimes = 0
        isGetBatteryVolt = False
        batteryVoltage = 0
        isRunning = False
        
        self.debug = debug
        if (self.debug):
            print("Reseting transmission")
    
    def getIsRunning(self):
        return isRunning
          
    def moveServo(self, servoId, position, time):
        buf = [0,0,0,0,0,0,0,0,0,0]
     
        if (servoId>=0 and servoId<=24 and time>0):
            buf[0] = self.__FRAME_HEADER             
            buf[1] = self.__FRAME_HEADER
            buf[2] = 8                             
            buf[3] = self.__CMD_SERVO_MOVE              
            buf[4] = 1                         
            buf[5] = getLowByte(time)        
            buf[6] = getHighByte(time)        
            buf[7] = servoId
            buf[8] = getLowByte(position)        
            buf[9] = getHighByte(position)
            
            self.ser.write(buf)

    def sendCMDGetBatteryVolt(self):
        buf = [0,0,0,0]
        buf[0] = self.__FRAME_HEADER
        buf[1] = self.__FRAME_HEADER
        buf[2] = 2
        buf[3] = self.__CMD_GET_BATTERY_VOLTAGE
        
        isGetBatteryVolt = false
        self.ser.write(buf)


if __name__=='__main__':

    while True:
        lsc = ServoController(debug=False)
        time.sleep(1)
        lsc.moveServo(0,2500,5000)
        time.sleep(1)
        lsc.moveServo(0,500,5000)


  
